@@ -1445,12 +1445,6 @@ static int activeDefragTimeProc(struct aeEventLoop *eventLoop, long long id, voi
     monotime endtime = starttime + dutyCycleUs;
     int haveMoreWork = 1;
 
-    /* Increment server.cronloops so that run_with_period works. */
-    long hz_ms = 1000 / server.hz;
-    int cronloops = (server.mstime - server.blocked_last_cron + (hz_ms - 1)) / hz_ms; /* rounding up */
-    server.blocked_last_cron += cronloops * hz_ms;
-    server.cronloops += cronloops;
-
     mstime_t latency;
     latencyStartMonitor(latency);
 
