@@ -2,13 +2,9 @@
  * Copyright (c) 2009-Present, Redis Ltd.
  * All rights reserved.
  *
- * Copyright (c) 2024-present, Valkey contributors.
- * All rights reserved.
- *
- * Licensed under your choice of the Redis Source Available License 2.0
- * (RSALv2) or the Server Side Public License v1 (SSPLv1).
- *
- * Portions of this file are available under BSD3 terms; see REDISCONTRIBUTIONS for more information.
+ * Licensed under your choice of (a) the Redis Source Available License 2.0
+ * (RSALv2); or (b) the Server Side Public License v1 (SSPLv1); or (c) the
+ * GNU Affero General Public License v3 (AGPLv3).
  */
 
 #include "script_lua.h"
@@ -51,6 +47,7 @@ static char *redis_api_allow_list[] = {
 static char *lua_builtins_allow_list[] = {
     "xpcall",
     "tostring",
+    "getfenv",
     "setmetatable",
     "next",
     "assert",
@@ -71,16 +68,15 @@ static char *lua_builtins_allow_list[] = {
     "loadstring",
     "ipairs",
     "_VERSION",
+    "setfenv",
     "load",
     "error",
     NULL,
 };
 
-/* Lua builtins which are deprecated for sandboxing concerns */
-static char *lua_builtins_deprecated[] = {
+/* Lua builtins which are not documented on the Lua documentation */
+static char *lua_builtins_not_documented_allow_list[] = {
     "newproxy",
-    "setfenv",
-    "getfenv",
     NULL,
 };
 
@@ -102,6 +98,7 @@ static char **allow_lists[] = {
     libraries_allow_list,
     redis_api_allow_list,
     lua_builtins_allow_list,
+    lua_builtins_not_documented_allow_list,
     lua_builtins_removed_after_initialization_allow_list,
     NULL,
 };
@@ -635,7 +632,10 @@ static void luaReplyToRedisReply(client *c, client* script_client, lua_State *lu
         lua_rawget(lua,-2);
         t = lua_type(lua,-1);
         if (t == LUA_TSTRING) {
-            addReplyStatusSafe(c, lua_tostring(lua,-1));
+            sds ok = sdsnew(lua_tostring(lua,-1));
+            sdsmapchars(ok,"\r\n","  ",2);
+            addReplyStatusLength(c, ok, sdslen(ok));
+            sdsfree(ok);
             lua_pop(lua,2);
             return;
         }
@@ -1301,22 +1301,7 @@ static int luaNewIndexAllowList(lua_State *lua) {
             break;
         }
     }
-
-    int allowed = (*allow_l != NULL);
-    /* If not explicitly allowed, check if it's a deprecated function. If so,
-     * allow it only if 'lua_enable_deprecated_api' config is enabled. */
-    int deprecated = 0;
-    if (!allowed) {
-        char **c = lua_builtins_deprecated;
-        for (; *c; ++c) {
-            if (strcmp(*c, variable_name) == 0) {
-                deprecated = 1;
-                allowed = server.lua_enable_deprecated_api ? 1 : 0;
-                break;
-            }
-        }
-    }
-    if (!allowed) {
+    if (!*allow_l) {
         /* Search the value on the back list, if its there we know that it was removed
          * on purpose and there is no need to print a warning. */
         char **c = deny_list;
@@ -1325,7 +1310,7 @@ static int luaNewIndexAllowList(lua_State *lua) {
                 break;
             }
         }
-        if (!*c && !deprecated) {
+        if (!*c) {
             serverLog(LL_WARNING, "A key '%s' was added to Lua globals which is not on the globals allow list nor listed on the deny list.", variable_name);
         }
     } else {
@@ -1377,37 +1362,6 @@ void luaSetTableProtectionRecursively(lua_State *lua) {
     }
 }
 
-/* Set the readonly flag on the metatable of basic types (string, nil etc.) */
-void luaSetTableProtectionForBasicTypes(lua_State *lua) {
-    static const int types[] = {
-        LUA_TSTRING,
-        LUA_TNUMBER,
-        LUA_TBOOLEAN,
-        LUA_TNIL,
-        LUA_TFUNCTION,
-        LUA_TTHREAD,
-        LUA_TLIGHTUSERDATA
-    };
-
-    for (size_t i = 0; i < sizeof(types) / sizeof(types[0]); i++) {
-        /* Push a dummy value of the type to get its metatable */
-        switch (types[i]) {
-            case LUA_TSTRING: lua_pushstring(lua, ""); break;
-            case LUA_TNUMBER: lua_pushnumber(lua, 0); break;
-            case LUA_TBOOLEAN: lua_pushboolean(lua, 0); break;
-            case LUA_TNIL: lua_pushnil(lua); break;
-            case LUA_TFUNCTION: lua_pushcfunction(lua, NULL); break;
-            case LUA_TTHREAD: lua_newthread(lua); break;
-            case LUA_TLIGHTUSERDATA: lua_pushlightuserdata(lua, (void*)lua); break;
-        }
-        if (lua_getmetatable(lua, -1)) {
-            luaSetTableProtectionRecursively(lua);
-            lua_pop(lua, 1); /* pop metatable */
-        }
-        lua_pop(lua, 1); /* pop dummy value */
-    }
-}
-
 void luaRegisterVersion(lua_State* lua) {
     lua_pushstring(lua,"REDIS_VERSION_NUM");
     lua_pushnumber(lua,REDIS_VERSION_NUM);
@@ -1626,9 +1580,6 @@ void luaExtractErrorInformation(lua_State *lua, errorInfo *err_info) {
     lua_getfield(lua, -1, "err");
     if (lua_isstring(lua, -1)) {
         err_info->msg = sdsnew(lua_tostring(lua, -1));
-    } else {
-        /* Ensure we never return a NULL msg. */
-        err_info->msg = sdsnew("ERR unknown error");
     }
     lua_pop(lua, 1);
 
@@ -1722,7 +1673,7 @@ void luaCallFunction(scriptRunCtx* run_ctx, lua_State *lua, robj** keys, size_t
                                       err_info.source,
                                       err_info.line);
             }
-            addReplyErrorSdsExSafe(c, final_msg, err_info.ignore_err_stats_update? ERR_REPLY_FLAG_NO_STATS_UPDATE : 0);
+            addReplyErrorSdsEx(c, final_msg, err_info.ignore_err_stats_update? ERR_REPLY_FLAG_NO_STATS_UPDATE : 0);
             luaErrorInformationDiscard(&err_info);
         }
         lua_pop(lua,1); /* Consume the Lua error */
