@@ -655,4 +655,50 @@ tags {"aof external:skip"} {
             }
         }
     }
+
+    start_server {overrides {loading-process-events-interval-bytes 1024}} {
+        test "Allow changing appendonly config while loading from AOF on startup" {
+            # Set AOF enabled, populate db and restart.
+            r config set appendonly yes
+            r config set key-load-delay 100
+            r config rewrite
+            populate 10000
+            restart_server 0 false false
+
+            # Disable AOF while loading from the disk.
+            assert_equal 1 [s loading]
+            r config set appendonly no
+            assert_equal 1 [s loading]
+
+            # Speed up loading, verify AOF disabled.
+            r config set key-load-delay 0
+            wait_done_loading r
+            assert_equal {10000} [r dbsize]
+            assert_equal 0 [s aof_enabled]
+        }
+
+        test "Allow changing appendonly config while loading from RDB on startup" {
+            # Set AOF disabled, populate db and restart.
+            r flushall
+            r config set appendonly no
+            r config set key-load-delay 100
+            r config rewrite
+            populate 10000
+            r save
+            restart_server 0 false false
+
+            # Enable AOF while loading from the disk.
+            assert_equal 1 [s loading]
+            r config set appendonly yes
+            assert_equal 1 [s loading]
+
+            # Speed up loading, verify AOF enabled, do a quick sanity.
+            r config set key-load-delay 0
+            wait_done_loading r
+            assert_equal {10000} [r dbsize]
+            assert_equal 1 [s aof_enabled]
+            r set t 1
+            assert_equal {1} [r get t]
+        }
+    }
 }
