@@ -1,4 +1,5 @@
 set testmodule [file normalize tests/modules/propagate.so]
+set miscmodule [file normalize tests/modules/misc.so]
 set keyspace_events [file normalize tests/modules/keyspace_events.so]
 
 tags "modules" {
@@ -45,8 +46,8 @@ tags "modules" {
                     $master set x y
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr notifications}
                         {set x y}
                         {exec}
@@ -63,8 +64,8 @@ tags "modules" {
                     $master exec
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr notifications}
                         {set x1 y1}
                         {incr notifications}
@@ -91,8 +92,8 @@ tags "modules" {
                     # Note whenever there's double notification: SET with PX issues two separate
                     # notifications: one for "set" and one for "expire"
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr notifications}
                         {incr notifications}
                         {set asdf1 1 PXAT *}
@@ -107,18 +108,24 @@ tags "modules" {
                         {incr notifications}
                         {set asdf3 3 PXAT *}
                         {exec}
+                        {multi}
                         {incr notifications}
                         {incr notifications}
                         {incr testkeyspace:expired}
                         {del asdf*}
+                        {exec}
+                        {multi}
                         {incr notifications}
                         {incr notifications}
                         {incr testkeyspace:expired}
                         {del asdf*}
+                        {exec}
+                        {multi}
                         {incr notifications}
                         {incr notifications}
                         {incr testkeyspace:expired}
                         {del asdf*}
+                        {exec}
                     }
                     close_replication_stream $repl
 
@@ -143,8 +150,11 @@ tags "modules" {
                     # Bottom line: "notifications" always exists and we can't really determine the order of evictions
                     # This test is here only for sanity
 
+                    # The replica will get the notification with multi exec and we have a generic notification handler
+                    # that performs `RedisModule_Call(ctx, "INCR", "c", "multi");` if the notification is inside multi exec.
+                    # so we will have 2 keys, "notifications" and "multi".
                     wait_for_condition 500 10 {
-                        [$replica dbsize] eq 1
+                        [$replica dbsize] eq 2 
                     } else {
                         fail "Not all keys have been evicted"
                     }
@@ -183,8 +193,8 @@ tags "modules" {
                     # Note that although CONFIG SET maxmemory is called in this flow (see issue #10014),
                     # eviction will happen and will not induce propagation of the CONFIG command (see #10019).
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr notifications}
                         {incr notifications}
                         {set asdf1 1 PXAT *}
@@ -199,12 +209,18 @@ tags "modules" {
                         {incr notifications}
                         {set asdf3 3 PXAT *}
                         {exec}
+                        {multi}
                         {incr notifications}
                         {del asdf*}
+                        {exec}
+                        {multi}
                         {incr notifications}
                         {del asdf*}
+                        {exec}
+                        {multi}
                         {incr notifications}
                         {del asdf*}
+                        {exec}
                         {multi}
                         {incr notifications}
                         {set asdf4 4}
@@ -229,8 +245,8 @@ tags "modules" {
                     }
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr notifications}
                         {incr notifications}
                         {set timer-maxmemory-volatile-start 1 PXAT *}
@@ -239,10 +255,14 @@ tags "modules" {
                         {incr notifications}
                         {set timer-maxmemory-volatile-end 1 PXAT *}
                         {exec}
+                        {multi}
                         {incr notifications}
                         {del timer-maxmemory-volatile-*}
+                        {exec}
+                        {multi}
                         {incr notifications}
                         {del timer-maxmemory-volatile-*}
+                        {exec}
                     }
                     close_replication_stream $repl
 
@@ -256,8 +276,8 @@ tags "modules" {
                     $master propagate-test.timer-eval
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr notifications}
                         {incrby timer-eval-start 1}
                         {incr notifications}
@@ -282,8 +302,8 @@ tags "modules" {
                     }
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incrby timer-nested-start 1}
                         {incrby timer-nested-end 1}
                         {exec}
@@ -307,8 +327,8 @@ tags "modules" {
                     }
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incrby timer-nested-start 1}
                         {incr notifications}
                         {incr using-call}
@@ -350,8 +370,8 @@ tags "modules" {
                     }
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr a-from-thread}
                         {incr notifications}
                         {incr thread-call}
@@ -385,8 +405,8 @@ tags "modules" {
                     }
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr thread-detached-before}
                         {incr notifications}
                         {incr thread-detached-1}
@@ -405,8 +425,8 @@ tags "modules" {
                     $master propagate-test.mixed
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr counter-1}
                         {incr counter-2}
                         {exec}
@@ -431,8 +451,8 @@ tags "modules" {
                         redis.call("propagate-test.mixed"); return "OK" } 0 ] {OK}
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr counter-1}
                         {incr counter-2}
                         {incr notifications}
@@ -456,8 +476,8 @@ tags "modules" {
                     $master propagate-test.mixed
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr counter-1}
                         {incr counter-2}
                         {exec}
@@ -482,8 +502,8 @@ tags "modules" {
                     $master propagate-test.mixed
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr counter-1}
                         {incr counter-2}
                         {exec}
@@ -515,8 +535,8 @@ tags "modules" {
                     }
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {incr counter-1}
                         {incr counter-2}
                         {incr notifications}
@@ -566,8 +586,8 @@ tags "modules" {
                     $master propagate-test.incr k1
 
                     assert_replication_stream $repl {
-                        {select *}
                         {multi}
+                        {select *}
                         {del k1}
                         {propagate-test.incr k1}
                         {exec}
@@ -580,6 +600,71 @@ tags "modules" {
                     assert_equal [$replica ttl k1] -1
                 }
 
+                test {module notification on set} {
+                    set repl [attach_to_replication_stream]
+
+                    $master SADD s foo
+
+                    wait_for_condition 500 10 {
+                        [$replica SCARD s] eq "1"
+                    } else {
+                        fail "Failed to wait for set to be replicated"
+                    }
+
+                    $master SPOP s 1
+
+                    wait_for_condition 500 10 {
+                        [$replica SCARD s] eq "0"
+                    } else {
+                        fail "Failed to wait for set to be replicated"
+                    }
+
+                    # Currently the `del` command comes after the notification.
+                    # When we fix spop to fire notification at the end (like all other commands),
+                    # the `del` will come first.
+                    assert_replication_stream $repl {
+                        {multi}
+                        {select *}
+                        {incr notifications}
+                        {sadd s foo}
+                        {exec}
+                        {multi}
+                        {incr notifications}
+                        {incr notifications}
+                        {del s}
+                        {exec}
+                    }
+                    close_replication_stream $repl
+                }
+
+                test {module key miss notification do not cause read command to be replicated} {
+                    set repl [attach_to_replication_stream]
+
+                    $master flushall
+                    
+                    $master get unexisting_key
+
+                    wait_for_condition 500 10 {
+                        [$replica get missed] eq "1"
+                    } else {
+                        fail "Failed to wait for set to be replicated"
+                    }
+
+                    # Test is checking a wrong!!! behavior that causes a read command to be replicated to replica/aof.
+                    # We keep the test to verify that such a wrong behavior does not cause any crashes.
+                    assert_replication_stream $repl {
+                        {select *}
+                        {flushall}
+                        {multi}
+                        {incr notifications}
+                        {incr missed}
+                        {get unexisting_key}
+                        {exec}
+                    }
+                    
+                    close_replication_stream $repl
+                }
+
                 test "Unload the module - propagate-test/testkeyspace" {
                     assert_equal {OK} [r module unload propagate-test]
                     assert_equal {OK} [r module unload testkeyspace]
@@ -591,8 +676,10 @@ tags "modules" {
     }
 }
 
+
 tags "modules aof" {
-    test {Modules RM_Replicate replicates MULTI/EXEC correctly} {
+    foreach aofload_type {debug_cmd startup} {
+    test "Modules RM_Replicate replicates MULTI/EXEC correctly: AOF-load type $aofload_type" {
         start_server [list overrides [list loadmodule "$testmodule"]] {
             # Enable the AOF
             r config set appendonly yes
@@ -606,11 +693,109 @@ tags "modules aof" {
             r propagate-test.mixed
             r exec
 
+            assert_equal [r get counter-1] {}
+            assert_equal [r get counter-2] {}
+            assert_equal [r get using-call] 2
+            assert_equal [r get after-call] 2
+            assert_equal [r get notifications] 4
+
             # Load the AOF
-            r debug loadaof
+            if {$aofload_type == "debug_cmd"} {
+                r debug loadaof
+            } else {
+                r config rewrite
+                restart_server 0 true false
+                wait_done_loading r
+            }
+
+            # This module behaves bad on purpose, it only calls
+            # RM_Replicate for counter-1 and counter-2 so values
+            # after AOF-load are different
+            assert_equal [r get counter-1] 4
+            assert_equal [r get counter-2] 4
+            assert_equal [r get using-call] 2
+            assert_equal [r get after-call] 2
+            # 4+4+2+2 commands from AOF (just above) + 4 "INCR notifications" from AOF + 4 notifications for these INCRs
+            assert_equal [r get notifications] 20
 
             assert_equal {OK} [r module unload propagate-test]
             assert_equal [s 0 unexpected_error_replies] 0
         }
     }
+    test "Modules RM_Call does not update stats during aof load: AOF-load type $aofload_type" {
+        start_server [list overrides [list loadmodule "$miscmodule"]] {
+            # Enable the AOF
+            r config set appendonly yes
+            r config set auto-aof-rewrite-percentage 0 ; # Disable auto-rewrite.
+            waitForBgrewriteaof r
+            
+            r config resetstat
+            r set foo bar
+            r EVAL {return redis.call('SET', KEYS[1], ARGV[1])} 1 foo bar2
+            r test.rm_call_replicate set foo bar3
+            r EVAL {return redis.call('test.rm_call_replicate',ARGV[1],KEYS[1],ARGV[2])} 1 foo set bar4
+            
+            r multi
+            r set foo bar5
+            r EVAL {return redis.call('SET', KEYS[1], ARGV[1])} 1 foo bar6
+            r test.rm_call_replicate set foo bar7
+            r EVAL {return redis.call('test.rm_call_replicate',ARGV[1],KEYS[1],ARGV[2])} 1 foo set bar8
+            r exec
+
+            assert_match {*calls=8,*,rejected_calls=0,failed_calls=0} [cmdrstat set r]
+            
+            
+            # Load the AOF
+            if {$aofload_type == "debug_cmd"} {
+                r config resetstat
+                r debug loadaof
+            } else {
+                r config rewrite
+                restart_server 0 true false
+                wait_done_loading r
+            }
+            
+            assert_no_match {*calls=*} [cmdrstat set r]
+            
+        }
+    }
+    }
+}
+
+# This test does not really test module functionality, but rather uses a module
+# command to test Redis replication mechanisms.
+test {Replicas that was marked as CLIENT_CLOSE_ASAP should not keep the replication backlog from been trimmed} {
+    start_server [list overrides [list loadmodule "$testmodule"]] {
+        set replica [srv 0 client]
+        start_server [list overrides [list loadmodule "$testmodule"]] {
+            set master [srv 0 client]
+            set master_host [srv 0 host]
+            set master_port [srv 0 port]
+            $master config set client-output-buffer-limit "replica 10mb 5mb 0"
+
+            # Start the replication process...
+            $replica replicaof $master_host $master_port
+            wait_for_sync $replica
+
+            test {module propagates from timer} {
+                # Replicate large commands to make the replica disconnected.
+                $master write [format_command propagate-test.verbatim 100000 [string repeat "a" 1000]] ;# almost 100mb
+                # Execute this command together with module commands within the same
+                # event loop to prevent periodic cleanup of replication backlog.
+                $master write [format_command info memory]
+                $master flush
+                $master read ;# propagate-test.verbatim
+                set res [$master read] ;# info memory
+
+                # Wait for the replica to be disconnected.
+                wait_for_log_messages 0 {"*flags=S*scheduled to be closed ASAP for overcoming of output buffer limits*"} 0 1500 10
+                # Due to the replica reaching the soft limit (5MB), memory peaks should not significantly
+                # exceed the replica soft limit. Furthermore, as the replica release its reference to
+                # replication backlog, it should be properly trimmed, the memory usage of replication
+                # backlog should not significantly exceed repl-backlog-size (default 1MB). */
+                assert_lessthan [getInfoProperty $res used_memory_peak] 10000000;# less than 10mb
+                assert_lessthan [getInfoProperty $res mem_replication_backlog] 2000000;# less than 2mb
+            }
+        }
+    }
 }
