@@ -1,4 +1,5 @@
-start_server {tags {"tracking network"}} {
+# logreqres:skip because it seems many of these tests rely heavily on RESP2
+start_server {tags {"tracking network logreqres:skip"}} {
     # Create a deferred client we'll use to redirect invalidation
     # messages to.
     set rd_redirection [redis_deferring_client]
@@ -229,22 +230,25 @@ start_server {tags {"tracking network"}} {
         # If a script doesn't call any read command, don't track any keys
         r EVAL "redis.call('set', 'key3{t}', 'bar')" 2 key1{t} key2{t} 
         $rd_sg MSET key2{t} 2 key1{t} 2
+        assert_equal "PONG" [r ping]
 
-        # If a script calls a read command, track all declared keys
-        r EVAL "redis.call('get', 'key3{t}')" 2 key1{t} key2{t} 
-        $rd_sg MSET key2{t} 2 key1{t} 2
+        # If a script calls a read command, just the read keys
+        r EVAL "redis.call('get', 'key2{t}')" 2 key1{t} key2{t}
+        $rd_sg MSET key2{t} 2 key3{t} 2
         assert_equal {invalidate key2{t}} [r read]
-        assert_equal {invalidate key1{t}} [r read]
+        assert_equal "PONG" [r ping]
 
         # RO variants work like the normal variants
-        r EVAL_RO "redis.call('ping')" 2 key1{t} key2{t} 
+
+        # If a RO script doesn't call any read command, don't track any keys
+        r EVAL_RO "redis.call('ping')" 2 key1{t} key2{t}
         $rd_sg MSET key2{t} 2 key1{t} 2
+        assert_equal "PONG" [r ping]
 
-        r EVAL_RO "redis.call('get', 'key1{t}')" 2 key1{t} key2{t} 
-        $rd_sg MSET key2{t} 3 key1{t} 3
+        # If a RO script calls a read command, just the read keys
+        r EVAL_RO "redis.call('get', 'key2{t}')" 2 key1{t} key2{t}
+        $rd_sg MSET key2{t} 2 key3{t} 2
         assert_equal {invalidate key2{t}} [r read]
-        assert_equal {invalidate key1{t}} [r read]
-
         assert_equal "PONG" [r ping]
     }
 
@@ -871,3 +875,28 @@ start_server {tags {"tracking network"}} {
     $rd_sg close
     $rd close
 }
+
+# Just some extra coverage for --log-req-res, because we do not
+# run the full tracking unit in that mode
+start_server {tags {"tracking network"}} {
+    test {Coverage: Basic CLIENT CACHING} {
+        set rd_redirection [redis_deferring_client]
+        $rd_redirection client id
+        set redir_id [$rd_redirection read]
+        assert_equal {OK} [r CLIENT TRACKING on OPTIN REDIRECT $redir_id]
+        assert_equal {OK} [r CLIENT CACHING yes]
+        r CLIENT TRACKING off
+    } {OK}
+
+    test {Coverage: Basic CLIENT REPLY} {
+        r CLIENT REPLY on
+    } {OK}
+
+    test {Coverage: Basic CLIENT TRACKINGINFO} {
+        r CLIENT TRACKINGINFO
+    } {flags off redirect -1 prefixes {}}
+
+    test {Coverage: Basic CLIENT GETREDIR} {
+        r CLIENT GETREDIR
+    } {-1}
+}
