@@ -1,17 +1,3 @@
-#
-# Copyright (c) 2009-Present, Redis Ltd.
-# All rights reserved.
-#
-# Copyright (c) 2024-present, Valkey contributors.
-# All rights reserved.
-#
-# Licensed under your choice of (a) the Redis Source Available License 2.0
-# (RSALv2); or (b) the Server Side Public License v1 (SSPLv1); or (c) the
-# GNU Affero General Public License v3 (AGPLv3).
-#
-# Portions of this file are available under BSD3 terms; see REDISCONTRIBUTIONS for more information.
-#
-
 # check functionality compression of plain and packed nodes
 start_server [list overrides [list save ""] ] {
     r config set list-compress-depth 2
@@ -1114,6 +1100,11 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         $watching_client get somekey{t}
         $watching_client read
         $watching_client exec
+        wait_for_condition 100 10 {
+            [regexp {cmd=exec} [r client list]] eq 1
+        } else {
+            fail "exec did not arrive"
+        }
         # Blocked BLPOPLPUSH may create problems, unblock it.
         r lpush srclist{t} element
         set res [$watching_client read]
@@ -2437,26 +2428,4 @@ foreach {pop} {BLPOP BLMPOP_RIGHT} {
         close_replication_stream $repl
     } {} {needs:repl}
 
-    test "Blocking timeout following PAUSE should honor the timeout" {
-        # cleanup first
-        r del mylist
-        
-        # create a test client
-        set rd [redis_deferring_client]
-        
-        # first PAUSE all writes for a very long time
-        r client pause 10000000000000 write
-
-        # block a client on the list
-        $rd BLPOP mylist 1
-        wait_for_blocked_clients_count 1
-        
-        # now unpause the writes
-        r client unpause
-
-        # client should time-out
-        wait_for_blocked_clients_count 0
-        
-        $rd close
-    }
 } ;# stop servers
